@@ -4,22 +4,32 @@ import gnu.math.IntNum;
 import gnu.math.DFloNum;
 import gnu.expr.*;
 import gnu.text.Char;
+import gnu.mapping.Procedure;
+import gnu.mapping.Values;
 
 /** Use to implement some special types that convert differently. */
 
-public class LangPrimType extends gnu.bytecode.PrimType
+public class LangPrimType extends PrimType implements TypeValue
 {
-  Interpreter interpreter;
+  Language language;
   PrimType implementationType;
 
-  public static LangPrimType byteType = new LangPrimType(Type.byte_type);
-  public static LangPrimType shortType = new LangPrimType(Type.short_type);
-  public static LangPrimType intType = new LangPrimType(Type.int_type);
-  public static LangPrimType longType = new LangPrimType(Type.long_type);
-  public static LangPrimType floatType = new LangPrimType(Type.float_type);
-  public static LangPrimType doubleType = new LangPrimType(Type.double_type);
-  public static LangPrimType charType = new LangPrimType(Type.char_type);
-  public static LangPrimType voidType = new LangPrimType(Type.void_type);
+  public static final LangPrimType byteType
+    = new LangPrimType(Type.byte_type);
+  public static final LangPrimType shortType
+    = new LangPrimType(Type.short_type);
+  public static final LangPrimType intType
+    = new LangPrimType(Type.int_type);
+  public static final LangPrimType longType
+    = new LangPrimType(Type.long_type);
+  public static final LangPrimType floatType
+    = new LangPrimType(Type.float_type);
+  public static final LangPrimType doubleType
+    = new LangPrimType(Type.double_type);
+  public static final LangPrimType charType
+    = new LangPrimType(Type.char_type);
+  public static final LangPrimType voidType
+    = new LangPrimType(Type.void_type);
 
   public LangPrimType (PrimType type)
   {
@@ -27,10 +37,10 @@ public class LangPrimType extends gnu.bytecode.PrimType
     implementationType = type;
   }
 
-  public LangPrimType (PrimType type, Interpreter interpreter)
+  public LangPrimType (PrimType type, Language language)
   {
     super(type);
-    this.interpreter = interpreter;
+    this.language = language;
     implementationType = type;
   }
 
@@ -40,11 +50,11 @@ public class LangPrimType extends gnu.bytecode.PrimType
   }
 
   public LangPrimType (String nam, String sig, int siz, Class reflectClass,
-		      Interpreter interpreter)
+		      Language language)
   {
     this(nam, sig, siz, reflectClass);
     implementationType = Type.signatureToPrimitive(sig.charAt(0));
-    this.interpreter = interpreter;
+    this.language = language;
   }
 
   public Type getImplementationType()
@@ -60,9 +70,11 @@ public class LangPrimType extends gnu.bytecode.PrimType
     switch (sig1)
       {
       case 'Z':
-	return interpreter.isTrue(obj) ? Boolean.TRUE : Boolean.FALSE;
+	return language.isTrue(obj) ? Boolean.TRUE : Boolean.FALSE;
       case 'C':
 	return new Character(((Char) obj).charValue());
+      case 'V':
+        return Values.empty;
       }
     return super.coerceFromObject(obj);
   }
@@ -74,13 +86,31 @@ public class LangPrimType extends gnu.bytecode.PrimType
     return  ((Char) value).charValue();
   }
 
+  public void emitIsInstance (CodeAttr code)
+  {
+    char sig1 = getSignature().charAt(0);
+    switch (sig1)
+      {
+      case 'Z':
+	code.emitPop(1);
+	code.emitPushInt(1);
+	break;
+      case 'C':
+	ClassType scmCharType = ClassType.make("gnu.text.Char");
+	code.emitInstanceof(scmCharType);
+	break;
+      default:
+	super.emitIsInstance(code);
+      }
+  }
+
   public void emitCoerceFromObject (CodeAttr code)
   {
     char sig1 = getSignature().charAt(0);
     switch (sig1)
       {
       case 'Z':
-	interpreter.emitCoerceToBoolean(code);
+	language.emitCoerceToBoolean(code);
 	break;
       case 'C':
 	// We handle char specially, because Kawa does not use standard
@@ -101,7 +131,7 @@ public class LangPrimType extends gnu.bytecode.PrimType
     switch (sig1)
       {
       case 'Z':
-	return interpreter.booleanObject(((Boolean) obj).booleanValue());
+	return language.booleanObject(((Boolean) obj).booleanValue());
       case 'C':
 	if (obj instanceof Char)
 	  return obj;
@@ -113,6 +143,7 @@ public class LangPrimType extends gnu.bytecode.PrimType
       case 'D':  case 'F':
 	return DFloNum.make(((Number) obj).doubleValue());
       case 'V':
+        // Perhaps we should return Language.noValue() instead?
 	return gnu.mapping.Values.empty;
       }
     return super.coerceToObject(obj);
@@ -127,9 +158,9 @@ public class LangPrimType extends gnu.bytecode.PrimType
       {
       case 'Z':
 	code.emitIfIntNotZero();
-	interpreter.emitPushBoolean(true, code);
+	language.emitPushBoolean(true, code);
 	code.emitElse();
-	interpreter.emitPushBoolean(false, code);
+	language.emitPushBoolean(false, code);
 	code.emitFi();
 	break;
       case 'C':
@@ -181,4 +212,36 @@ public class LangPrimType extends gnu.bytecode.PrimType
       return 1;
     return super.compare(other);
   }
+
+  public void emitTestIf(Variable incoming, Declaration decl, Compilation comp)
+  {
+    char sig1 = getSignature().charAt(0);
+    /*
+    switch (sig1)
+      {
+      case 'Z':
+      }
+    */
+    CodeAttr code = comp.getCode();
+    if (incoming != null)
+      code.emitLoad(incoming);
+    if (decl != null)
+      {
+	code.emitDup();
+	decl.compileStore(comp);
+      }
+    emitIsInstance(code);
+    code.emitIfIntNotZero();
+  }
+
+  public void emitIsInstance(Variable incoming,
+			     Compilation comp, Target target)
+  {
+    gnu.kawa.reflect.InstanceOf.emitIsInstance(this, incoming, comp, target);
+  }
+
+  public Procedure getConstructor ()
+  {
+    return null;
+  }
 }
