@@ -1,8 +1,12 @@
 (define as-xml (make <gnu.kawa.xml.OutputAsXML>))
 
+(define-constant comment <gnu.kawa.xml.KComment>)
+
+(define-constant processing-instruction <gnu.kawa.xml.KProcessingInstruction>)
+
 ;;; Parse an XML file (specified by a URL or url string), giving a <document>.
 (define (parse-xml-from-url url) :: <document>
-  (invoke-static <gnu.kawa.xml.Document> 'document url))
+  (invoke-static <gnu.kawa.xml.Document> 'parse url))
 
 #|
 (define (parse-nsxml-from-url url) :: <document>
