@@ -26,4 +26,10 @@ public class KComment extends KCharacterData
     return "#comment";
   }
 
+  public static KComment valueOf (String text)
+  {
+    NodeTree tree = new NodeTree();
+    tree.writeComment(text, 0, text.length());
+    return new KComment(tree, 0);
+  }
 }
