@@ -1,4 +1,4 @@
-// Copyright (c) 2001  Per M.A. Bothner and Brainfood Inc.
+// Copyright (c) 2001, 2002  Per M.A. Bothner and Brainfood Inc.
 // This is free software;  for terms and warranty disclaimer see ./COPYING.
 
 package gnu.lists;
@@ -6,7 +6,11 @@ import java.io.*;
 
 /** Simple adjustable-length vector of unsigned 64-bit integers (longs). */
 
-public class U64Vector extends SimpleVector implements Externalizable
+public class U64Vector extends SimpleVector
+  implements Externalizable
+  /* #ifdef JAVA2 */
+  , Comparable
+  /* #endif */
 {
   long[] data;
 
@@ -123,7 +127,7 @@ public class U64Vector extends SimpleVector implements Externalizable
 
   public String getTag() { return "u64"; }
 
-  public boolean consumeNext(int ipos, Object xpos, Consumer out)
+  public boolean consumeNext (int ipos, Consumer out)
   {
     int index = ipos >>> 1;
     if (index >= size)
@@ -132,17 +136,36 @@ public class U64Vector extends SimpleVector implements Externalizable
     return true;
   }
 
-  public void consume(int iposStart, Object xposStart,
-		      int iposEnd, Object xposEnd, Consumer out)
+  public void consumePosRange (int iposStart, int iposEnd, Consumer out)
   {
     if (out.ignoring())
       return;
     int i = iposStart >>> 1;
     int end = iposEnd >>> 1;
+    if (end > size)
+      end = size;
     for (;  i < end;  i++)
       out.writeLong(data[i]);
   }
 
+  public int compareTo(Object obj)
+  {
+    U64Vector vec2 = (U64Vector) obj;
+    long[] arr1 = data;
+    long[] arr2 = vec2.data;
+    int n1 = size;
+    int n2 = vec2.size;
+    int n = n1 > n2 ? n2 : n1;
+    for (int i = 0;  i < n;  i++)
+      {
+	long v1 = arr1[i];
+	long v2 = arr2[i];
+	if (v1 != v2)
+	  return (v1^0x8000000000000000L) > (v2^0x8000000000000000L) ? 1 : -1;
+      }
+    return n1 - n2;
+  }
+
   /**
    * @serialData Write 'size' (using writeInt),
    *   followed by 'size' elements in order (using writeLong).
