@@ -1,6 +1,7 @@
 package gnu.mapping;
 import java.io.*;
 import gnu.lists.*;
+import gnu.text.Printable;
 
 /** Encapsulate multiple values in a single object.
  * In Scheme and Lisp mainly used to return multiple values from a function.
@@ -78,44 +79,62 @@ public class Values extends TreeList implements Printable, Externalizable
 	|| (next = list.nextDataIndex(startPosition)) <= 0)
       return empty;
     if (next == endPosition || list.nextDataIndex(next) < 0)
-      return list.getNext(startPosition << 1, null); // Singleton value
+      return list.getPosNext(startPosition << 1); // Singleton value
     Values vals = new Values();
-    list.consumeRange(startPosition, endPosition, vals);
+    list.consumeIRange(startPosition, endPosition, vals);
     return vals;
   }
 
+  /** If a simple value, return that value.
+   * Also, if no values, return empty.
+   */
+  public final Object canonicalize ()
+  {
+    if (gapEnd == data.length)
+      {
+	if (gapStart == 0)
+	  return empty;
+	if (nextDataIndex(0) == gapStart) // Singleton value.
+	  return getPosNext(0);
+      }
+    return this;
+  }
+
   /** Apply a Procedure with these values as the arguments. */
   public Object call_with (Procedure proc) throws Throwable
   {
     return proc.applyN (toArray());
   }
 
-  public boolean equals (Object obj)
-  {
-    if (obj instanceof Values)
-      return super.equals(obj);
-    if (size() != 0)
-      return false;
-    Object x = getNext(0, null);
-    return x != null && x.equals(obj);
-  }
-
-  public void print(java.io.PrintWriter ps)
+  public void print (Consumer out)
   {
     if (this == empty)
       {
-	ps.print("#!void");
+	out.write("#!void");
 	return;
       }
     Object[] vals = toArray();  // FIXME!
     int size = vals.length;
-    ps.print("#<values");
-    for (int i = 0; i < size; i++)
+    boolean readable = true;  // FIXME
+    if (readable)
+      out.write("#<values");
+    for (int i = 0;;)
       {
-	ps.print (" ");
-	SFormat.print (vals[i], ps);
+        int next = nextDataIndex(i);
+        if (next < 0)
+          break;
+	out.write(' ');
+        if (i >= gapEnd)
+          i -= gapEnd - gapStart;
+        Object val = getPosNext(i << 1);
+        if (val instanceof Printable)
+          ((Printable) val).print(out);
+        else
+          out.writeObject(val);
+        i = next;
       }
-    ps.print (">");
+    if (readable)
+      out.write('>');
   }
 
   /**
@@ -166,7 +185,12 @@ public class Values extends TreeList implements Printable, Externalizable
   public static Object nextValue(Object values, int curIndex)
   {
     if (values instanceof Values)
-      return ((Values) values).getNext(curIndex << 1, null);
+      {
+	Values v = (Values) values;
+	if (curIndex >= v.gapEnd)
+	  curIndex -= v.gapEnd - v.gapStart;
+	return ((Values) values).getPosNext(curIndex << 1);
+      }
     else
       return values;
   }
@@ -191,4 +215,9 @@ public class Values extends TreeList implements Printable, Externalizable
     else
       out.writeObject(value);
   }
+
+  public static int countValues (Object value)
+  {
+    return value instanceof Values ? ((Values) value).size() : 1;
+  }
 }
