@@ -1,4 +1,4 @@
-// Copyright (c) 2001, 2004  Per M.A. Bothner.
+// Copyright (c) 2001, 2004, 2006  Per M.A. Bothner.
 // This is free software;  for terms and warranty disclaimer see ./COPYING.
 
 package gnu.text;
@@ -19,20 +19,20 @@ public class PrettyWriter extends java.io.Writer
   public PrettyWriter(java.io.Writer out)
   {
     this.out = out;
-    isPrettyPrinting = true;
+    prettyPrintingMode = 1;
   }
 
   public PrettyWriter(java.io.Writer out, int lineLength)
   {
     this.out = out;
     this.lineLength = lineLength;
-    isPrettyPrinting = lineLength > 1;
+    prettyPrintingMode = lineLength > 1 ? 1 : 0;
   }
 
-  public PrettyWriter(java.io.Writer out, boolean isPrettyPrinting)
+  public PrettyWriter(java.io.Writer out, boolean prettyPrintingMode)
   {
     this.out = out;
-    this.isPrettyPrinting = isPrettyPrinting;
+    this.prettyPrintingMode = prettyPrintingMode ? 1 : 0;
   }
 
   /** Line length we should format to. */
@@ -46,7 +46,34 @@ public class PrettyWriter extends java.io.Writer
   public static ThreadLocation indentLoc
     = new ThreadLocation("indent");
 
-  public boolean isPrettyPrinting;
+  /** The current pretty-printing mode.
+   * See setPrettyPrintingMode for valid values. */
+  int prettyPrintingMode;
+
+  /** Control pretty-printing mode.
+   * @param mode the value 0 disables pretty-printing;
+   *   the value 1 enables ecplicit pretty-printing;
+   *   the value 2 enables pretty-printing with auto-fill, which means that
+   *   spaces are treated like enqueing NEWLINE_SPACE (essentiall a 'fill').
+   */
+  public void setPrettyPrintingMode (int mode)
+  { prettyPrintingMode = mode; }
+
+  /** Return pretty-printing mode.
+   * @return 0, 1, 2, as described for {@link #setPrettyPrintingMode(int)}.
+   */
+  public int getPrettyPrintingMode () { return prettyPrintingMode; }
+
+  /** Is pretty printing enabled? */
+  public boolean isPrettyPrinting () { return prettyPrintingMode > 0; }
+
+  /** Turn pretty printing on or off.
+   * Equivalent to {@code setPrettyPrintingMode(mode?1:0)}.
+   */
+  public void setPrettyPrinting (boolean mode)
+  {
+    prettyPrintingMode = mode ? 0 : 1;
+  }
 
   public static int initialBufferSize = 126;
 
@@ -94,8 +121,10 @@ public class PrettyWriter extends java.io.Writer
   }
 
   /** Stack of logical blocks in effect at the buffer start.
-   * I.e. blocks for which reallyStartLogicalBlock has been called. */
+   * I.e. blocks for which {@code reallyStartLogicalBlock} has been called.
+   * Each block uses {@code LOGICAL_BLOCK_LENGTH} {@code int} in this array. */
   int[] blocks = new int[10 * LOGICAL_BLOCK_LENGTH];
+  /** Number of {@code int}s used by each block in the {@code blocks} array. */
   static final private int LOGICAL_BLOCK_LENGTH = 6;
   static final private int BLOCK_START_COLUMN = -1;
   static final private int BLOCK_SECTION_COLUMN = -2;
@@ -103,6 +132,7 @@ public class PrettyWriter extends java.io.Writer
   static final private int BLOCK_PREFIX_LENGTH = -4;
   static final private int BLOCK_SUFFIX_LENGTH = -5;
   static final private int BLOCK_SECTION_START_LINE = -6;
+  /** The "stack pointer" in the {@code blocks} array. */
   int blockDepth = LOGICAL_BLOCK_LENGTH;
 
   /** Buffer holding the per-line prefix active at the buffer start.
@@ -246,10 +276,35 @@ public class PrettyWriter extends java.io.Writer
     return blocks[blockDepth+BLOCK_SECTION_START_LINE];
   }
 
+  boolean wordEndSeen;
+
+  /** Note the end of a "word".  See {@link #writeWordStart}. */
+  public void writeWordEnd ()
+  {
+    wordEndSeen = true;
+  }
+
+  /** Maybe write a word-separating space.
+   * Specifically, write a space if the previous output
+   * was {@link #writeWordEnd}.  Otherwise, do nothing.
+   */
+  public void writeWordStart ()
+  {
+    if (wordEndSeen)
+      write(' ');
+    wordEndSeen = false;
+  }
+
+  public void clearWordEnd ()
+  {
+    wordEndSeen = false;
+  }
+
   public void write (int ch)
   {
+    wordEndSeen = false;
     //log("{WRITE-ch: "+((char)ch)+"}");
-    if (ch == '\n' && isPrettyPrinting)
+    if (ch == '\n' && prettyPrintingMode > 0)
       enqueueNewline(NEWLINE_LITERAL);
     else
       {
@@ -257,7 +312,7 @@ public class PrettyWriter extends java.io.Writer
 	int fillPointer = bufferFillPointer;
 	buffer[fillPointer] = (char) ch;
 	bufferFillPointer = 1 + fillPointer;
-	if (ch == ' ' && isPrettyPrinting && currentBlock < 0)
+	if (ch == ' ' && prettyPrintingMode > 1 && currentBlock < 0)
 	  enqueueNewline(NEWLINE_SPACE);
       }
   }
@@ -269,6 +324,7 @@ public class PrettyWriter extends java.io.Writer
 
   public void write (String str, int start, int count)
   {
+    wordEndSeen = false;
     //log("{WRITE-str: "+str.substring(start, start+count)+"}");
     while (count > 0)
       {
@@ -282,7 +338,7 @@ public class PrettyWriter extends java.io.Writer
 	while (--cnt >= 0)
 	  {
 	    char ch = str.charAt(start++);
-	    if (ch == '\n' && isPrettyPrinting)
+	    if (ch == '\n' && prettyPrintingMode > 0)
 	      {
 		bufferFillPointer = fillPointer;
 		enqueueNewline(NEWLINE_LITERAL);
@@ -291,7 +347,7 @@ public class PrettyWriter extends java.io.Writer
 	    else
 	      {
 		buffer[fillPointer++] = (char) ch;
-		if (ch == ' ' && isPrettyPrinting && currentBlock < 0)
+		if (ch == ' ' && prettyPrintingMode > 1 && currentBlock < 0)
 		  {
 		    bufferFillPointer = fillPointer;
 		    enqueueNewline(NEWLINE_SPACE);
@@ -310,6 +366,7 @@ public class PrettyWriter extends java.io.Writer
 
   public void write (char[] str, int start, int count)
   {
+    wordEndSeen = false;
     //log("{WRITE: "+new String(str, start, count)+"}");
     int end = start + count;
   retry:
@@ -319,7 +376,7 @@ public class PrettyWriter extends java.io.Writer
 	for (int i = start;  i < end;  i++)
 	  {
 	    char c;
-	    if (isPrettyPrinting
+	    if (prettyPrintingMode > 0
 		&& ((c = str[i]) == '\n'
 		    || (c == ' ' && currentBlock < 0)))
 	      {
@@ -505,6 +562,7 @@ public class PrettyWriter extends java.io.Writer
 
   public void enqueueNewline (int kind)
   {
+    wordEndSeen = false;
     int depth = pendingBlocksCount;
     //enqueueExtraLog = " kind:"+(char) kind;
     int newline = enqueue(QITEM_NEWLINE_TYPE, QITEM_NEWLINE_SIZE);
@@ -534,13 +592,12 @@ public class PrettyWriter extends java.io.Writer
 	todo -= size;
 	entry += size;
       }
-    maybeOutput (kind == NEWLINE_LITERAL
-		 || kind == NEWLINE_MANDATORY);
+    maybeOutput (kind == NEWLINE_LITERAL || kind == NEWLINE_MANDATORY, false);
   }
 
   public final void writeBreak(int kind)
   {
-    if (isPrettyPrinting)
+    if (prettyPrintingMode > 0)
       enqueueNewline(kind);
   }
 
@@ -555,7 +612,7 @@ public class PrettyWriter extends java.io.Writer
 
   public void addIndentation(int amount, boolean current)
   {
-    if (isPrettyPrinting)
+    if (prettyPrintingMode > 0)
       enqueueIndent((current ? QITEM_INDENTATION_CURRENT
 		     : QITEM_INDENTATION_BLOCK),
 		    amount);
@@ -586,7 +643,7 @@ public class PrettyWriter extends java.io.Writer
       }
     if (prefix != null)
       write(prefix);
-    if (! isPrettyPrinting)
+    if (prettyPrintingMode == 0)
       return;
     int start = enqueue (QITEM_BLOCK_START_TYPE,
 			 QITEM_BLOCK_START_SIZE);
@@ -664,7 +721,7 @@ public class PrettyWriter extends java.io.Writer
 
   public void endLogicalBlock (String suffix)
   {
-    if (isPrettyPrinting)
+    if (prettyPrintingMode > 0)
       endLogicalBlock();
     else if (suffix != null)
       write(suffix);
@@ -816,9 +873,9 @@ public class PrettyWriter extends java.io.Writer
     int available = length - fillPtr;
     if (available > 0)
       return available;
-    else if (isPrettyPrinting && fillPtr > lineLength)
+    else if (prettyPrintingMode > 0 && fillPtr > lineLength)
       {
-	if (! maybeOutput(false))
+	if (! maybeOutput(false, false))
 	  outputPartialLine();
 	return ensureSpaceInBuffer(want);
       }
@@ -833,7 +890,7 @@ public class PrettyWriter extends java.io.Writer
       }
   }
 
-  boolean maybeOutput(boolean forceNewlines) // DONE
+  boolean maybeOutput(boolean forceNewlines, boolean flushing)
   {
     boolean outputAnything = false;
     //log("maybeOutput("+forceNewlines+"):");  dumpQueue();
@@ -848,6 +905,7 @@ public class PrettyWriter extends java.io.Writer
 	  {
 	  case QITEM_NEWLINE_TYPE:
 	    boolean cond;
+            int fits = -1;
 	    switch (queueInts[next+QITEM_NEWLINE_KIND])
 	      {
 	      default: // LINEAR, LITERAL, or MANDATORY:
@@ -874,10 +932,10 @@ public class PrettyWriter extends java.io.Writer
 		    if (end >= queueInts.length)
 		      end -= queueInts.length;
 		  }
-		int fits = fitsOnLine(end, forceNewlines);
+		fits = fitsOnLine(end, forceNewlines);
 		if (fits > 0)
 		  cond = false;
-		else if (fits < 0)
+		else if (fits < 0 || flushing)
 		  cond = true;
 		else
 		  break loop;
@@ -888,7 +946,10 @@ public class PrettyWriter extends java.io.Writer
 		outputAnything = true;
 		try
 		  {
-		    outputLine(next);
+                    if (flushing && fits == 0)
+                      outputPartialLine();
+                    else
+                      outputLine(next);
 		  }
 		catch (IOException ex)
 		  {
@@ -914,7 +975,7 @@ public class PrettyWriter extends java.io.Writer
 	    int end = queueInts[next + QITEM_SECTION_START_SECTION_END];
 	    // Convert relative offset to absolute index:
 	    end = end > 0 ? (end + next) % queueInts.length : -1;
-	    int fits = fitsOnLine (end, forceNewlines);
+	    fits = fitsOnLine (end, forceNewlines);
 	    //log("block-start @"+next+" end:"+end+" force:"+forceNewlines+" fits:"+fits);
 	    if (fits > 0)
 	      {
@@ -928,7 +989,7 @@ public class PrettyWriter extends java.io.Writer
 		queueSize -= endr;
 		//log("remove block -> next:"+next+" endr:"+endr+" qSize:"+queueSize);
 	      }
-	    else if (fits < 0)
+	    else if (fits < 0 || flushing)
 	      {
 		String prefix = queueStrings[next + QITEM_BLOCK_START_PREFIX];
 		String suffix = queueStrings[next + QITEM_BLOCK_START_SUFFIX];
@@ -1005,7 +1066,7 @@ public class PrettyWriter extends java.io.Writer
     // Hook.
   }
 
-  /** Output a newline.
+  /** Output a new line.
    * @param newline index of a newline queue item
    */
   void outputLine (int newline)  throws IOException
@@ -1084,7 +1145,7 @@ public class PrettyWriter extends java.io.Writer
 
   void outputPartialLine ()
   {
-    int fillPtr = bufferFillPointer;
+    //log("outputPartialLine");
     int tail = queueTail;
     while (queueSize > 0 && getQueueType(tail) == QITEM_NOP_TYPE)
       {
@@ -1095,6 +1156,7 @@ public class PrettyWriter extends java.io.Writer
 	  tail = 0;
 	queueTail = tail;
       }
+    int fillPtr = bufferFillPointer;
     int count = queueSize > 0 ? posnIndex (queueInts[tail + QITEM_POSN])
       : fillPtr;
     int newFillPtr = fillPtr - count;
@@ -1117,7 +1179,9 @@ public class PrettyWriter extends java.io.Writer
 
   public void forcePrettyOutput () throws IOException
   {
-    maybeOutput(false);
+    maybeOutput(false, true);
+    if (bufferFillPointer > 0)
+      outputPartialLine();
     expandTabs(-1);
     bufferStartColumn = getColumnNumber();
     out.write(buffer, 0, bufferFillPointer);
@@ -1151,7 +1215,7 @@ public class PrettyWriter extends java.io.Writer
     buffer = null;
   }
 
-  /** Not meaningful if isPrettyPrinting. */
+  /** Not meaningful if {@code prettyPrintingMode > 0}. */
   public int getColumnNumber ()
   {
     int i = bufferFillPointer;
@@ -1161,7 +1225,7 @@ public class PrettyWriter extends java.io.Writer
 	  return bufferStartColumn + bufferFillPointer;
 	char ch = buffer[i];
 	if (ch == '\n' || ch == '\r')
-	  return bufferFillPointer - i;
+	  return bufferFillPointer - (i+1);
       }
   }
 
@@ -1294,6 +1358,10 @@ public class PrettyWriter extends java.io.Writer
 	    out.print(skind);
 	    out.println(')');
 	    break;
+          case QITEM_INDENTATION_TYPE:
+	    printQueueWord(start, QITEM_INDENTATION_KIND, "kind", out);
+	    printQueueWord(start, QITEM_INDENTATION_AMOUNT, "amount", out);
+	    break;
 	  default:
 	    for (int i = 2;  i < size;  i++)
 	      printQueueWord(start, i, "word#"+i, out);
