@@ -12,6 +12,7 @@ import gnu.mapping.ThreadLocation;
 import gnu.mapping.Symbol;
 import java.math.BigDecimal;
 import gnu.expr.Keyword;
+import gnu.kawa.xml.XmlNamespace;
 
 /** Print an event stream in XML format on a PrintWriter. */
 
@@ -39,7 +40,7 @@ public class XMLPrinter extends OutPort
    * 0: No element element tags, as required for canonical XML:
    * {@code <br></br>}.
    * 1: Use XML-style empty element tags: {@code <br/>}
-   * 1: Use HTML-compatible empty element tags: {@code <br />}
+   * 2: Use HTML-compatible empty element tags: {@code <br />}
    */
   public int useEmptyElementTag = 2;
   public boolean escapeText = true;
@@ -59,6 +60,8 @@ public class XMLPrinter extends OutPort
   public static final ThreadLocation indentLoc
     = new ThreadLocation("xml-indent");
 
+  public boolean strict;
+
   /** Chain of currently active namespace nodes. */
   NamespaceBinding namespaceBindings = NamespaceBinding.predefinedXML;
 
@@ -134,13 +137,27 @@ public class XMLPrinter extends OutPort
       {
 	isHtml = true;
 	useEmptyElementTag = 2;
+        // Pre-establish the html namespace, so it doesn't get printed.
+        if (namespaceBindings == NamespaceBinding.predefinedXML)
+          namespaceBindings = XmlNamespace.HTML_BINDINGS;
       }
+    else if (namespaceBindings == XmlNamespace.HTML_BINDINGS)
+      namespaceBindings = NamespaceBinding.predefinedXML;
     if ("xhtml".equals(style))
       useEmptyElementTag = 2;
     if ("plain".equals(style))
       escapeText = false;
   }
 
+  boolean mustHexEscape (int v)
+  {
+    return (v >= 127 && (v <= 159 || escapeNonAscii))
+      || v == 0x2028
+      // We must escape control characters in attributes,
+      // since otherwise they get normalized to ' '.
+      || (v < ' ' && (inAttribute || (v != '\t' && v != '\n')));
+  }
+
   public void write (int v)
   {
     closeTag();
@@ -184,10 +201,7 @@ public class XMLPrinter extends OutPort
 	  bout.write("&amp;");
 	else if (v == '\"' && inAttribute)
 	  bout.write("&quot;");
-	else if ((escapeNonAscii && v >= 127)
-                 // We must escape control characters in attributes,
-                 // since otherwise they get normalized to ' '.
-                 || (v < ' ' && (inAttribute || (v != '\t' && v != '\n'))))
+	else if (mustHexEscape(v))
           {
             int i = v;
             if (v >= 0xD800)
@@ -424,6 +438,8 @@ public class XMLPrinter extends OutPort
 	    if (uri == namespaceBindings.resolve(prefix))
 	      // A matching namespace declaration is already in scope.
 	      continue;
+            if (uri == null && prefix != null && ! undeclareNamespaces)
+              continue;
 	    bout.write(' '); // prettyprint break
 	    if (prefix == null)
 	      bout.write("xmlns");
@@ -545,6 +561,8 @@ public class XMLPrinter extends OutPort
    * This is only allowed immediately after a startElement. */
   public void startAttribute (Object attrType)
   {
+    if (! inStartTag && strict)
+      error("attribute not in element", "SENR0001");
     if (inAttribute)
       bout.write('"');
     inAttribute = true;
@@ -724,7 +742,7 @@ public class XMLPrinter extends OutPort
         while (start < limit)
           {
             char c = str.charAt(start++);
-            if (c >= 127 || c == '\n' || c == '\r'
+            if (mustHexEscape(c)
                 || (inComment > 0 ? (c == '-' || inComment == 2)
                     : (c == '<' || c == '>' || c == '&'
                        || (inAttribute && (c == '"' || c < ' ' )))))
@@ -753,7 +771,7 @@ public class XMLPrinter extends OutPort
         while (off < limit)
           {
             char c = buf[off++];
-            if (c >= 127 || c == '\n' || c == '\r'
+            if (mustHexEscape(c)
                 || (inComment > 0 ? (c == '-' || inComment == 2)
                     : (c == '<' || c == '>' || c == '&'
                        || (inAttribute && (c == '"' || c < ' ' )))))
@@ -862,4 +880,9 @@ public class XMLPrinter extends OutPort
   {
     position.sequence.consumeNext(position.ipos, this);
   }
+
+  public void error (String msg, String code)
+  {
+    throw new RuntimeException("serialization error: "+msg+" ["+code+']');
+  }
 }
