@@ -598,7 +598,7 @@ public class XQuery extends Language
     defProcStFld("iterator-items",
 		 "gnu.kawa.xml.IteratorItems", "iteratorItems");
     defProcStFld("list-items", "gnu.kawa.xml.ListItems", "listItems");
-    define_method("node-name", "gnu.kawa.xml.NodeName", "nodeName");
+    define_method("node-name", "gnu.xquery.util.NodeUtils", "nodeName");
     define_method("nilled", "gnu.xquery.util.NodeUtils", "nilled");
     define_method("data", "gnu.xquery.util.NodeUtils", "data$X");
     define_method("lower-case", "gnu.xquery.util.StringUtils", "lowerCase");
@@ -706,6 +706,14 @@ public class XQuery extends Language
     define_method("adjust-time-to-timezone", "gnu.xquery.util.TimeUtils",
                   "adjustTimeToTimezone"); // overloaded
     define_method("dateTime", "gnu.xquery.util.TimeUtils", "dateTime");
+    define_method("current-dateTime", "gnu.xquery.util.TimeUtils",
+                  "currentDateTime");
+    define_method("current-date", "gnu.xquery.util.TimeUtils",
+                  "currentDate");
+    define_method("current-time", "gnu.xquery.util.TimeUtils",
+                  "currentTime");
+    define_method("implicit-timezone", "gnu.xquery.util.TimeUtils",
+                  "implicitTimezone");
 
     define_method("zero-or-one", "gnu.xquery.util.SequenceUtils", "zeroOrOne");
     define_method("one-or-more", "gnu.xquery.util.SequenceUtils", "oneOrMore");
@@ -802,7 +810,16 @@ public class XQuery extends Language
       "base64Binary", XDataType.base64BinaryType,
       "NOTATION", XDataType.NotationType,
       "QName", "gnu.mapping.Symbol",
-      "anyAtomicType", Type.pointer_type
+      "normalizedString", XStringType.normalizedStringType,
+      "token", XStringType.tokenType,
+      "language", XStringType.languageType,
+      "NMTOKEN", XStringType.NMTOKENType,
+      "Name", XStringType.NameType,
+      "NCName", XStringType.NCNameType,
+      "ID", XStringType.IDType,
+      "IDREF", XStringType.IDREFType,
+      "ENTITY", XStringType.ENTITYType,
+      "anyAtomicType", XDataType.anyAtomicType
     };
 
   public static Type getStandardType (String name)
@@ -830,6 +847,17 @@ public class XQuery extends Language
     return t != null ? t : Scheme.string2Type(name);
   }
 
+  public String formatType (Type type)
+  {
+    String tname = type.getName();
+    if ("gnu.math.IntNum".equals(tname))
+      return "xs:integer";
+    if ("java.lang.String".equals(tname)
+        || "java.lang.CharSequence".equals(tname))
+      return "xs:string";
+    return type.toString();
+  }
+
   public Type getTypeFor (Class clas)
   {
     if (clas.isPrimitive())
@@ -843,7 +871,7 @@ public class XQuery extends Language
       {
         String name = clas.getName();
         if (name.equals("java.lang.String"))
-          return XDataType.stringType;
+          return XDataType.stringStringType;
         if (name.equals("gnu.kawa.xml.UntypedAtomic"))
           return XDataType.untypedAtomicType;
         if (name.equals("java.lang.Boolean"))
