@@ -24,12 +24,14 @@ public class RunXQTS extends FilterConsumer
   // HashMap<String,Object> sources = new HashMap<String,Object>();
   // Stack<Symbol> externalVariablesSet = new Stack<Symbol>();
   // Stack<String> outputFileAlts = new Stack<String>();
+  // Stack<String> outputCompareAlts = new Stack<String>();
   /* #else */
   Hashtable expectedFailures = new Hashtable();
   Hashtable modules = new Hashtable();
   Hashtable sources = new Hashtable();
   Stack externalVariablesSet = new Stack();
   Stack outputFileAlts = new Stack();
+  Stack outputCompareAlts = new Stack();
   /* #endif */
   ModuleManager manager = ModuleManager.getInstance();
   Object failExpected;
@@ -45,7 +47,6 @@ public class RunXQTS extends FilterConsumer
   String XQueryXQueryOffsetPath;
   String XQueryFileExtension;
   String XQueryXFileExtension;
-  String compare;
   Object contextItem;
 
   int passCount;
@@ -210,14 +211,14 @@ public class RunXQTS extends FilterConsumer
     this.cout = out;
 
     expectFailures("K-ReplaceFunc-8", "allow bad regex replacement string");
-    expectFailures("Constr-namespace-13", "testsuite error? missing namespace undeclaration");
+    expectFailures("K2-MatchesFunc-1", "allow bad regex pattern");
     expectFailures("static-context-1", "unchecked unknownType in element(*,TypeName)");
-    expectFailures("NodTest003", "actually pass? different char encoding");
-    expectFailures("K-FunctionProlog-11|K-FunctionProlog-41",
-                   "item() is treated as equivalent to item()*");
-    expectFailures("ForExprType030|ForExprType033|LocalNameFromQNameFunc005|"
-                   +"CastAs671|CastAs672",
-                   "xs:normalizedString, xs:NCName, xs:ENTITY not implemented");
+    expectFailures("fn-abs-more-args-023|fn-abs-more-args-024",
+                   "testsuite error (4023): -0 is not  valid unsignedLong/unsignedShort literal");
+    expectFailures("ST-WhereExpr001",
+                   "testsuite error (4024) - should not require error");
+    expectFailures("K2-SeqExprCast-207",
+                   "was testsuite error(4252) - now mismatch between '>' and '&gt;'");
     /* #ifndef JAVA5 */
     expectFailures("surrogates12|surrogates13|surrogates14|surrogates15",
                    "surrogates not handled by java.util.regex");
@@ -268,13 +269,9 @@ public class RunXQTS extends FilterConsumer
     expectFailures("K-SubstringBeforeFunc-5|K-SubstringAfterFunc-5|"
                    +"K-ContainsFunc-5|K-StartsWithFunc-5|K-EndsWithFunc-5",
                    "some string functions don't support collation argument");
-    expectFailures("K-CodepointToStringFunc-8|K-CodepointToStringFunc-11|"
-                   +"K-CodepointToStringFunc-12|K-CodepointToStringFunc-14|"
-                   +"K-CodepointToStringFunc-15",
-                   "test-case excessively strict about disallowed characetrs");
     expectFailures("caselessmatch04",
                    "regex/unicode special case");
-    expectFailures("string-queries-results-q4|K2-FunctionProlog-7",
+    expectFailures("string-queries-results-q4|K2-FunctionProlog-7|K2-FunctionProlog-17|K2-FunctionProlog-18|K2-FunctionProlog-19|K2-FunctionProlog-22",
                    "function conversion incorrect for user-defined functions");
     expectFailures("caselessmatch10|caselessmatch11",
                    // Need to translate [xxx-[yyy]] to [xxx&&[^yyy]].
@@ -396,6 +393,7 @@ public class RunXQTS extends FilterConsumer
         testFilePath = attributes.getValue("FilePath");
         testQueryName = null;
         outputFileAlts.clear();
+        outputCompareAlts.clear();
         expectedErrorsBuf.setLength(1);
         manager.clear();
       }
@@ -582,6 +580,7 @@ public class RunXQTS extends FilterConsumer
     gnu.lists.Consumer save = ctx.consumer;
     CharArrayOutPort out = new CharArrayOutPort();
     XMLPrinter xout = new XMLPrinter(out, false);
+    xout.strict = true;
     xout.useEmptyElementTag = 1;
     xout.escapeNonAscii = false;
     xout.canonicalizeCDATA = true;
@@ -647,8 +646,16 @@ public class RunXQTS extends FilterConsumer
     boolean foundMatchingOutput = false;
     boolean displayDifference = false;
     String expected = null;
+    String compare = null;
     for (int ialt = 0;  ialt < numOutputFileAlts;  ialt++)
       {
+        compare = (String) outputCompareAlts.elementAt(ialt);
+        if ("Ignore".equals(compare))
+          {
+            report("pass", null);
+            foundMatchingOutput = true;
+            break;
+          }
         String outname  = directory + '/' + ResultOffsetPath + testFilePath
           + outputFileAlts.elementAt(ialt);
         FileInputStream expectStream = new FileInputStream(outname);
@@ -702,7 +709,7 @@ public class RunXQTS extends FilterConsumer
           }
       }
     if (displayDifference)
-      {
+      { // This only displays a single expected result.
         writeVerbose("compare", compare);
         writeVerbose("expected", expected);
         writeVerbose("actual", actual);
@@ -742,7 +749,54 @@ public class RunXQTS extends FilterConsumer
           return true;
         int c1 = i1 == len1 ? -1 : arg1.charAt(i1);
         int c2 = i2 == len2 ? -1 : arg2.charAt(i2);
-        if (c1 == c2)
+        if ((c1 == '&' && arg1.charAt(i1+1) == '#' && i1 + 3 < len1)
+            || (c2 == '&' && arg2.charAt(i2+1) == '#' && i2 + 3 < len2))
+          {
+            try
+              {
+                if (c1 == '&')
+                  {
+                    int base1 = 10;
+                    i1 += 2;
+                    if (arg1.charAt(i1) == 'x')
+                      {
+                        i1++;
+                        base1 = 16;
+                      }
+                    int semi1 = arg1.indexOf(';', i1);
+                    c1 = Integer.parseInt(arg1.substring(i1, semi1), base1);
+                    i1 = semi1;
+                  }
+                else if (c1 >= 0xD800 && c1 < 0xDC00 && i1 + 1 < len1)
+                  c1 = (c1 - 0xD800) * 0x400
+                    + (arg1.charAt(++i1) - 0xDC00) + 0x10000;
+                if (c2 == '&')
+                  {
+                    int base2 = 10;
+                    i2 += 2;
+                    if (arg2.charAt(i2) == 'x')
+                      {
+                        i2++;
+                        base2 = 16;
+                      }
+                    int semi2 = arg2.indexOf(';', i2);
+                    c2 = Integer.parseInt(arg2.substring(i2, semi2), base2);
+                    i2 = semi2;
+                  }
+                else if (c2 >= 0xD800 && c2 < 0xDC00 && i2 + 1 < len2)
+                  c2 = (c2 - 0xD800) * 0x400
+                    + (arg2.charAt(++i2) - 0xDC00) + 0x10000;
+              }
+            catch (Throwable ex)
+              {
+                return false;
+              }
+            if (c1 != c2)
+              return false;
+            i1++;
+            i2++;
+          }
+        else if (c1 == c2)
           {
             if (c1 == '<' && isXML)
               {
@@ -898,33 +952,6 @@ public class RunXQTS extends FilterConsumer
             i1 = i1 + 2;
             i2 = i2 + 1;
           }
-        else if (c2 == '&' && i2 + 2 < len2
-                 && arg2.charAt(i2+1) == '#')
-          {
-            if (c1 >= 0xD800 && c1 < 0xDC00 && i1 + 1 < len1)
-              c1 = (c1 - 0xD800) * 0x400
-                  + (arg1.charAt(++i1) - 0xDC00) + 0x10000;
-            i2 = i2 + 2;
-            int base = 10;
-            if (arg2.charAt(i2) == 'x')
-              {
-                i2++;
-                base = 16;
-              }
-            int semi = arg2.indexOf(';', i2);
-            try
-              {
-                c2 = Integer.parseInt(arg2.substring(i2, semi), base);
-              }
-            catch (Throwable ex)
-              {
-                return false;
-              }
-            i1 = i1 + 1;
-            i2 = semi+1;
-            if (c1 != c2)
-              return false;
-          }
         else
           return false;
       }
@@ -1092,7 +1119,7 @@ public class RunXQTS extends FilterConsumer
     else if (tagMatches("output-file"))
       {
         outputFileAlts.push(getElementValue());
-        compare = attributes.getValue("compare");
+        outputCompareAlts.push(attributes.getValue("compare"));
       }
     else if (tagMatches("test-suite"))
       {
