@@ -1,6 +1,5 @@
 package gnu.xquery.testsuite;
 import gnu.expr.*;
-import kawa.*;
 import gnu.mapping.*;
 import gnu.xquery.lang.*;
 import gnu.text.*;
@@ -21,11 +20,11 @@ public class TestMisc
   public static void main(String[] args)
   {
     // gnu.expr.ModuleExp.dumpZipPrefix = "kawa-zip-dump-";
-    // gnu.expr.ModuleExp.debugPrintExpr = true;
+    // Compilation.debugPrintExpr = true;
     // Compilation.debugPrintFinalExpr = true;
 
     evalTest("3.5+1", "4.5");
-    evalTest("3.5+1 ,4*2.5", "4.5 10.0");
+    evalTest("3.5+1 ,4*2.5", "4.5 10");
     evalTest("3<5", "true");
     evalTest("let $x:=3+4 return $x", "7");
     evalTest("let $x:=3+4 return <a>{$x}</a>", "<a>7</a>");
@@ -118,9 +117,6 @@ public class TestMisc
 
     evalTest("(doc('tab.xml')/result/row/*)[2]",
 	     "<fld2"+tabNsNodes+" align=\"right\"><!--ignore-this-comment-->12</fld2>");
-    evalTest("(doc('tab.xml')/result/row/*)[2 to 3]",
-	     "<fld2"+tabNsNodes+" align=\"right\"><!--ignore-this-comment-->12</fld2>"
-	     +" <fld1"+tabNsNodes+" align=\"left\">b1</fld1>");
     evalTest("(doc('tab.xml')/result/row/*)[position()>1]",
 	     "<fld2"+tabNsNodes+" align=\"right\"><!--ignore-this-comment-->12</fld2>"
 	     +"<fld1"+tabNsNodes+" align=\"left\">b1</fld1>"
@@ -134,12 +130,12 @@ public class TestMisc
 	     +"<fld1"+tabNsNodes+" align=\"left\">b1</fld1>"
 	     +"<fld2"+tabNsNodes+" align=\"right\">22</fld2>");
 
-    evalTest("string-value(doc('tab.xml'))",
+    evalTest("string(doc('tab.xml'))",
 	     "\n\na1\n12\n\n\nb1\n22\n\n\nc1\n33\n44\nc2\n\n\n");
     evalTest("string(doc('tab.xml'))",
 	     "\n\na1\n12\n\n\nb1\n22\n\n\nc1\n33\n44\nc2\n\n\n");
     evalTest("string(doc('tab.xml')/result/row/fld1/@align)", "left");
-    evalTest("string(doc('tab.xml')/result/row/fld2/@align)",
+    evalTest("doc('tab.xml')/result/row/fld2/@align/string()",
 	     "rightright");
  
     evalTest("for $x in children(<a>xy{3+4}kl<c>def</c>{9}{11}</a>)" +
@@ -192,14 +188,14 @@ public class TestMisc
 	     + " return ('[', $n, ']')",
 	     "[<a at1=\"val1\"><b /><c /></a>][ at1=\"val1\"][<b />][<c />]");
 
-    // Boundary whitsapce (xmlspace) tests:
-    evalTest("declare xmlspace preserve;\n"
+    // Boundary whitsapce (boundary-space) tests:
+    evalTest("declare boundary-space preserve;\n"
 	     + "for $n in (<a> <b/> {' x '} </a>)/node() return ($n,';')",
 	     " ;<b/>;  x  ;");
-    evalTest("declare xmlspace skip;\n"
+    evalTest("declare boundary-space strip;\n"
 	     + "for $n in (<a> <b/> {' x '} </a>)/node() return ($n,';')",
 	     "<b/>; x ;");
-    evalTest("declare xmlspace skip;\n"
+    evalTest("declare boundary-space strip;\n"
 	     + "for $n in (<a> x <b/> y<c/>&#x20;</a>)/node() return ($n,';')",
 	     " x ;<b/>; y;<c/>; ;");
     evalTest("for $n in (<a> <b/> </a>)/node() return ($n,';')",
@@ -224,22 +220,22 @@ public class TestMisc
     evalTest("declare namespace xx='XXX';\n <xx:a>XX</xx:a>",
 	     "<xx:a xmlns:xx=\"XXX\">XX</xx:a>");
     evalTest("declare namespace x1='XXX';\n declare namespace x2='XXX';\n"
-	     + "string((<top><x1:ab>X1</x1:ab><x2:ab>X2</x2:ab></top>)/x2:ab)",
+	     + "(<top><x1:ab>X1</x1:ab><x2:ab>X2</x2:ab></top>)/x2:ab/string()",
 	     "X1X2");
     evalTest("declare namespace x1='XXX';\n declare namespace x2='YYY';\n"
 	     + "string((<top><x1:ab>X1</x1:ab><x2:ab>X2</x2:ab></top>)/x2:ab)",
 	     "X2");
     evalTest("declare namespace x1='XXX';\n declare namespace x2='YYY';\n"
-	     + "string((<top><x1:ab>X1</x1:ab><x2:ab>X2</x2:ab></top>)/*)",
+	     + "(<top><x1:ab>X1</x1:ab><x2:ab>X2</x2:ab></top>)/*/string()",
 	     "X1X2");
     evalTest("declare namespace x1='XXX';\n declare namespace x2='YYY';\n"
-	     + "string((<top><x1:ab>X1</x1:ab><x2:ab>X2</x2:ab></top>)/*:*)",
+	     + "(<top><x1:ab>X1</x1:ab><x2:ab>X2</x2:ab></top>)/*:*/string()",
 	     "X1X2");
     evalTest("declare namespace x1='XXX';\n declare namespace x2='YYY';\n"
 	     + "string((<top><x1:ab>X1</x1:ab><x2:ab>X2</x2:ab></top>)/x1:*)",
 	     "X1");
     evalTest("declare namespace x1='XXX';\n declare namespace x2='YYY';\n"
-	     + "string((<top><x1:ab>X1</x1:ab><x2:ab>X2</x2:ab></top>)/*:ab)",
+	     + "(<top><x1:ab>X1</x1:ab><x2:ab>X2</x2:ab></top>)/*:ab/string()",
 	     "X1X2");
     evalTest("declare namespace x1='XXX';\n declare namespace x2='YYY';\n"
 	     + "string((<top><x1:ab>X1</x1:ab><x2:cd>X2</x2:cd></top>)/*:cd)",
@@ -248,14 +244,14 @@ public class TestMisc
 	     + "string(doc('tab.xml')/result/h:row)",
 	     "\nc1\n33\n44\nc2\n");
     evalTest("declare namespace xx='H';\n"
-	     + "string(doc('tab.xml')/result/xx:row)",
+	     + "doc('tab.xml')/result/xx:row/string()",
 	     "\nc1\n33\n44\nc2\n");
-    evalTest("string(doc('tab.xml')/result/*:row)",
+    evalTest("doc('tab.xml')/result/*:row/string()",
 	     "\na1\n12\n\nb1\n22\n\nc1\n33\n44\nc2\n");
-    evalTest("string(doc('tab.xml')/result/*:row/*:fld1)",
+    evalTest("doc('tab.xml')/result/*:row/*:fld1/string()",
 	     "a1b1c1c2");
     evalTest("declare namespace k='J';\n"
-	     + "string(doc('tab.xml')/result/*:row/k:fld1)",
+	     + "doc('tab.xml')/result/*:row/k:fld1/string()",
 	     "c1c2");
     evalTest("declare namespace k='J';\n"
 	     + "string(doc('tab.xml')/result/*:row[k:fld1])",
@@ -263,7 +259,7 @@ public class TestMisc
     evalTest("declare namespace m1 = 'bb'; declare namespace m2 = 'cc';"
 	     + "let $m1:x := 3 return let $m2:x := 4 return"
 	     + "  <m2:a a:c='{$a:x}' xmlns:a='bb'>{ count($a:x) }</m2:a>",
-	     "<m2:a xmlns:m2=\"cc\" xmlns:a=\"bb\" a:c=\"3\">1</m2:a>");
+	     "<m2:a xmlns:a=\"bb\" xmlns:m2=\"cc\" a:c=\"3\">1</m2:a>");
 
     evalTest("doc('tab.xml')/result/row[1]/descendant::*",
 	     "<fld1"+tabNsNodes+">a1</fld1>"
@@ -291,7 +287,7 @@ public class TestMisc
 	     " ;  ;  ;  ; j ; h ; j ; k ;");
     evalTest("for $n in doc('tab.xml')/result/*:row/*:fld1 "
 	     + "return <n>{namespace-uri-from-QName(node-name($n))}</n>",
-	     "<n></n><n></n><n>J</n><n>J</n>");
+	     "<n /><n /><n>J</n><n>J</n>");
     evalTest("for $n in doc('tab.xml')/result/*:row/*:fld1 return "
 	     + "('[', for $p in ('', 'k', 'h') return"
 	     + " (namespace-uri-for-prefix($p,$n),';'), ']')",
@@ -324,9 +320,9 @@ public class TestMisc
 	     "true true true");
     evalTest("for $n in children(<a>xx<b/>yy</a>) return $n instance of text ( )",
 	     "true false true");
-    evalTest("for $n in children(<a>xx<b/>yy</a>) return $n instance of element(a,*)",
+    evalTest("for $n in children(<a>xx<b/>yy</a>) return $n instance of element(a)",
 	     "false false false");
-    evalTest("for $n in <a>xx<b/>yy</a>/node() return $n instance of element(b,*)",
+    evalTest("for $n in <a>xx<b/>yy</a>/node() return $n instance of element(b)",
 	     "false true false");
     // FIXME: evalTest("<a>xx<b/>yy</a>/node() instance of node()", "false");
     evalTest("<a>xx<b/>yy</a>/node() instance of node()?", "false");
@@ -379,11 +375,11 @@ public class TestMisc
     evalNodeNames(some_elements+"($b, $a, $b, $d) except ()", "a;b;d;");
 
     // Check for catching errors:
-    evalTest("+ +", "*** syntax error - <string>:1:3: missing expression");
+    evalTest("+ +", "*** syntax error - <string>:1:4: missing expression [XPST0003]");
 
     evalTest("declare namespace x1='XXX",
 	     "*** caught SyntaxException - <string>:1:22: "
-	     + "unexpected end-of-file in string starting here");
+	     + "unexpected end-of-file in string starting here [XPST0003]");
 
     evalTest("unescaped-data('<?--->'),let $x:=unescaped-data('an &amp;oslash;') return <b>{unescaped-data('<![CDATA[saw]]>')} {$x}</b>",
 	     "<?---><b><![CDATA[saw]]> an &oslash;</b>");
@@ -538,6 +534,7 @@ public class TestMisc
     CharArrayOutPort wr = new CharArrayOutPort();
     gnu.xml.XMLPrinter xp = new gnu.xml.XMLPrinter(wr);
     xp.writeObject(value);
+    xp.close();
     return wr.toString();
   }
 
@@ -554,7 +551,7 @@ public class TestMisc
 	      returned = throwable;
 	  }
 	throwable = (Throwable) returned;
-	// throwable.printStackTrace();
+        // throwable.printStackTrace();
 	if (returned instanceof SyntaxException)
 	  result = "*** caught SyntaxException - "
 	    + ((SyntaxException) returned).getMessages().getErrors();
@@ -601,7 +598,7 @@ public class TestMisc
     InPort in = new CharArrayInPort(expr);
     SourceMessages messages = new SourceMessages();
 
-    Compilation comp = interp.parse(in, messages, interp.PARSE_IMMEDIATE);
+    Compilation comp = interp.parse(in, messages, Language.PARSE_IMMEDIATE);
     SourceError firstError = messages.getErrors();
     if (firstError != null)
       return "*** syntax error - " + firstError;
@@ -614,14 +611,16 @@ public class TestMisc
 	ctx.consumer = interp.getOutputConsumer(out);
 	ModuleExp mod = comp.getModule();
 	mod.setName("atInteractiveLevel");  // FIXME
-	ModuleExp.evalModule(env, ctx, comp);
+	ModuleExp.evalModule(env, ctx, comp, null, null);
       }
     finally
       {
 	ctx.consumer = save;
       }
 
-    return new String(out.toCharArray());
+    String result = new String(out.toCharArray());
+    out.close();
+    return result;
   }
 
 }
