@@ -8,8 +8,10 @@ import gnu.mapping.*;
 import gnu.xml.TextUtils;
 import gnu.kawa.xml.KNode;
 import gnu.kawa.xml.UntypedAtomic;
+/* #ifdef use:java.util.regex */
 import java.util.regex.Pattern;
 import java.util.regex.Matcher;
+/* #endif */
 import gnu.text.*;
 /* #ifdef use:java.text.Normalizer */
 // import java.text.Normalizer;
@@ -27,11 +29,15 @@ public class StringUtils
     if ((arg == Values.empty || arg == null) && onEmpty != ERROR_VALUE)
       return onEmpty;
     if (arg instanceof UntypedAtomic 
-        || arg instanceof Path
+        /* #ifdef use:java.lang.CharSequence */
+        || arg instanceof CharSequence
+        /* #else */
+        // || arg instanceof String
+        /* #endif */
         /* #ifdef use:java.net.URI */
         || arg instanceof java.net.URI
         /* #endif */
-        || arg instanceof String)
+        || arg instanceof Path)
       return arg.toString();
     throw new WrongType(functionName, iarg, arg,
                         onEmpty == ERROR_VALUE ? "xs:string" : "xs:string?");
@@ -282,7 +288,9 @@ public class StringUtils
 
   public static String concat$V (Object arg1, Object arg2, Object[] args)
   {
+    arg1 = SequenceUtils.coerceToZeroOrOne(arg1, "concat", 1);
     String str1 = TextUtils.stringValue(arg1);
+    arg2 = SequenceUtils.coerceToZeroOrOne(arg2, "concat", 2);
     String str2 = TextUtils.stringValue(arg2);
     /* #ifdef JAVA5 */
     // StringBuilder result = new StringBuilder(str1);
@@ -292,7 +300,10 @@ public class StringUtils
     result.append(str2);
     int count = args.length;
     for (int i = 0; i < count; i++)
-      result.append(TextUtils.stringValue(args[i]));
+      {
+        Object arg = SequenceUtils.coerceToZeroOrOne(args[i], "concat", i+2);
+        result.append(TextUtils.stringValue(arg));
+      }
     return result.toString();
   }
 
