@@ -1,21 +1,24 @@
 package kawa.standard;
 import kawa.lang.*;
-import gnu.bytecode.Access;
 import gnu.bytecode.ClassType;
 import gnu.bytecode.Type;
-import gnu.mapping.Binding;
 import gnu.expr.*;
 import gnu.lists.*;
 
-// OPC: (primitive-op1 OPC "rettype"  ("argtype" ...))
-// 182: (primitive-virtual-method "class" "method" "rettype" ("argtype" ...))
-// 183: (primitive-constructor "class" ("argtype ...))
-// 184: (primitive-static-method "class" "method" "rettype" ("argtype" ...))
-// 185: (primitive-interface-method "class" "method" "rettype" ("argtype" ...))
-
 public class prim_method extends Syntax
 {
-  static private Pattern pattern2 = new ListPat (2);
+  public static final prim_method virtual_method = new prim_method(182);
+  static { virtual_method.setName("primitive-virtual-method"); }
+
+  public static final prim_method static_method = new prim_method(184);
+  static { static_method.setName("primitive-static-method"); }
+
+  public static final prim_method interface_method = new prim_method(185);
+  static { interface_method.setName("primitive-interface-method"); }
+
+  public static final prim_method op1 = new prim_method();
+  static { op1.setName("primitive-op1"); }
+
   static private Pattern pattern3 = new ListPat (3);
   static private Pattern pattern4 = new ListPat (4);
 
@@ -32,27 +35,10 @@ public class prim_method extends Syntax
   {
   }
 
-  public static Type exp2Type (Object obj, Translator tr)
-  {
-    String str = obj.toString();
-    if (obj instanceof FString)
-      return Scheme.string2Type(str);
-    else if (obj instanceof String || obj instanceof Binding)
-      {
-        Type type = tr.getInterpreter().getTypeFor(tr.rewrite(obj));
-	if (type != null)
-	  return type;
-      }
-    tr.syntaxError("invalid type spec (must be \"type\" or 'type or <type>): "
-                   + str);
-    return Type.pointer_type;
-  }
-
   public Expression rewrite (Object obj, Translator tr)
   {
     Object[] match = new Object [4];
     if (! (op_code == 0 ? pattern3.match(obj, match, 1)
-	   : op_code == 183 ? pattern2.match(obj, match, 2) // constructor
 	   : pattern4.match(obj, match, 0))) // virtual or static
       return tr.syntaxError ("wrong number of arguments to "+getName()
 			     +"(opcode:"+op_code+")");
@@ -66,10 +52,10 @@ public class prim_method extends Syntax
     for (int i = 0;  i < narg;  i++)
       {
 	Pair p = (Pair)argp;
-	args[i] = exp2Type(p.car, tr);
+	args[i] = tr.exp2Type(p);
 	argp = (LList)p.cdr;
       }
-    Type rtype = exp2Type(match[2], tr);
+    Type rtype = tr.exp2Type(new Pair(match[2], null));
     PrimProcedure proc;
     if (op_code == 0)
       {
@@ -79,18 +65,7 @@ public class prim_method extends Syntax
     else
       {
         ClassType cl = null;
-        Type ctype;
-        int carg;
-        if (op_code == 183)
-          {
-            carg = 2;
-            ctype = rtype;
-          }
-        else
-          {
-            carg = 0;
-            ctype = exp2Type(match[0], tr);
-          }
+        Type ctype = tr.exp2Type((Pair) obj);
         try
           {
             cl = (ClassType) ctype;
@@ -106,21 +81,14 @@ public class prim_method extends Syntax
                 code = 'w';
                 ((ClassType) cl).setExisting(false);
               }
-            tr.error(code, "unknown class: " + match[carg]);
-          }
-        if (op_code == 183)  // primitive-constructor
-          {
-            proc = new PrimProcedure(cl, args);
-          }
-        else
-          {
-            Pair p;
-            if (match[1] instanceof Pair
-                && (p = (Pair) match[1]).car == "quote")
-              match[1] = ((Pair) p.cdr).car;
-            proc = new PrimProcedure(op_code, cl,
-                                     match[1].toString(), rtype, args);
+            tr.error(code, "unknown class: " + match[0]);
           }
+        Pair p;
+        if (match[1] instanceof Pair
+            && (p = (Pair) match[1]).car == "quote")
+          match[1] = ((Pair) p.cdr).car;
+        proc = new PrimProcedure(op_code, cl,
+                                 match[1].toString(), rtype, args);
       }
     return new QuoteExp(proc);
   }
