@@ -189,14 +189,9 @@ public class require extends Syntax
   public static ModuleInfo lookupModuleFromSourcePath (String sourceName, ScopeExp defs)
   {
     ModuleManager manager = ModuleManager.getInstance();
-    try
-      {
-        sourceName = URI_utils.resolve(sourceName, defs.getFileName()).toString();
-      }
-    catch (java.net.URISyntaxException ex)
-      {
-        return null;
-      }
+    String baseName = defs.getFileName();
+    if (baseName != null)
+      sourceName = Path.valueOf(baseName).resolve(sourceName).toString();
     return manager.findWithSourcePath(sourceName);
   }
 
@@ -221,7 +216,7 @@ public class require extends Syntax
         Compilation comp;
         try
           {
-            InPort fstream = InPort.openFile(info.sourceAbsPath);
+            InPort fstream = InPort.openFile(info.getSourceAbsPath());
             info.clearClass();
             info.className = className;
             comp = language.parse(fstream, messages, info);
