@@ -1,4 +1,4 @@
-(test-init "Miscellaneous" 142)
+(test-init "Miscellaneous" 151)
 
 ;;; DSSSL spec example 11
 (test '(3 4 5 6) (lambda x x) 3 4 5 6)
@@ -167,11 +167,13 @@
 (test #t eof-object? (peek-char str-inport))
 (close-input-port str-inport)
 
-(test "a(b c)" 'open-output-string
+(test "a/b (c d)" 'open-output-string
       (let ((q (open-output-string))
-            (x '(a b c)))
+            (x '(a b c d)))
         (write (car x) q)
-        (write (cdr x) q)
+	(display "/" q)
+        (write (cadr x) q)
+        (write (cddr x) q)
         (get-output-string q)))
 
 ;;; From: Hallvard Traetteberg <Hallvard.Traetteberg@idi.ntnu.no>
@@ -659,6 +661,16 @@
     (format #f "x = ~S." x)))
 (test "x = -4." test-neg-abs)
 
+(test '((prefix-test 11)
+	(prefix-test:var2 12)
+	(prefix-test:var2:var3 13)
+	(prefix-test:filler:var4 14))
+      'prefix-test
+      prefix-test-list)
+(test '(12) 'prefix-test:var2 prefix-test:var2)
+(test '(13) 'prefix-test:var2:var3 prefix-test:var2:var3)
+(test '(14) 'prefix-test:filler:var4 prefix-test:filler:var4)
+
 ;; Common Lisp hyperspec
 (test "[#24rn]" 'print-base-1 ;; Common Lisp returns upper-case #24rN
       (fluid-let ((*print-base* 24) (*print-radix* #t))
@@ -678,3 +690,25 @@
 	     (fluid-let ((*print-radix* #t) (*print-base* pb))
 	       (format #f "~S ~S" 10 1/10)))
 	   '(2 3 8 10 16)))
+
+;; Savannah bug #14697 Error using :: <int>
+;; Submitted by:  	Gerardo Horvilleur <mago>
+(define bug14697-result "")
+(let ((GS.261 :: <int> 10)
+      (GS.262 :: <int> 1))
+  (do ((i :: <int> 1 (+ i GS.262)))
+      ((> i GS.261))
+    (set! bug14697-result (string-append bug14697-result " "
+					 (number->string i)))))
+(test " 1 2 3 4 5 6 7 8 9 10" 'bug14697 bug14697-result)
+
+(require 'xml)
+(test "<code xmlns=\"http://www.w3.org/1999/xhtml\">Foo</code>" 'html-contructor-1
+      (as-xml (html:code "Foo")))
+(test "<a xmlns=\"http://www.w3.org/1999/xhtml\" href=\"foo.html\">Foo</a>" 'html-contructor-2
+      (as-xml (html:a href:"foo.html" "Foo")))
+(define-xml-namespace h "HTML")
+(test "<h:code xmlns:h=\"HTML\">Foo</h:code>" 'html-contructor-3
+      (as-xml (h:code "Foo")))
+(test "<b xmlns=\"http://www.w3.org/1999/xhtml\"><code>Foo</code></b>" 'html-contructor-4
+      (as-xml (html:b (html:code "Foo"))))
