@@ -24,7 +24,7 @@
 ;;
 ;; test-report:  Called at end to print a summary.
 ;;
-;; fail-unexpected:  If non-null, if means the following test is
+;; fail-expected:  If non-false, if means the following test is
 ;; expected to fail.  The actual value should be string explaining
 ;; the failure.  For example:
 ;; (set! fail-expected "sqrt of negative number not supported")
@@ -84,8 +84,8 @@
 (define (section . args)
   (set! cur-section args)
   (display-section (or *log-file* (current-output-port)))
-  (set! last-section #f)
-  #t)
+  (set! last-section #f))
+
 (define record-error (lambda (e) (set! errs (cons (list cur-section e) errs))))
 
 (define (report-pass port fun args res)
@@ -96,8 +96,11 @@
   (newline port))
 
 (define (report-fail port fun args res expect)
-  (display (if fail-expected (string-append "XFAIL (" fail-expected "): ")
-	       "FAIL: ") port)
+  (display (cond ((string? fail-expected)
+		  (string-append "XFAIL (" fail-expected "): "))
+		 (fail-expected "XFAIL: ")
+		 (else "FAIL: "))
+	   port)
   (write (cons fun args) port)
   (display "  ==> " port)
   (write res port)
@@ -116,14 +119,12 @@
 ;; want to use '+ in the test function below.  The clean (future) solution
 ;; would be to put this file inside a module.  FIXME.
 
-(define save-+ +)
-
 (define (test expect fun . args)
   ((lambda (res)
      (cond ((equal? expect res)
 	    (if fail-expected
 		(set! xpass-count (+ xpass-count 1))
-		(set! pass-count (save-+ pass-count 1)))
+		(set! pass-count (+ pass-count 1)))
 	    (if *log-file*
 		(report-pass *log-file* fun args res))
 	    (cond ((or verbose fail-expected)
@@ -139,7 +140,16 @@
 		   (maybe-report-section)
 		   (report-fail (current-output-port) fun args res expect)))))
      (set! fail-expected #f))
-   (if (procedure? fun) (apply fun args) (car args))))
+   (if (procedure? fun)
+       (cond-expand (kawa
+		     (try-catch
+		      (apply fun args)
+		      (ex <java.lang.Throwable>
+			  ;; (invoke ex 'printStackTrace) ;; for DEBUGGING
+			  ex)))
+		    (else
+		     (apply fun args)))
+       (car args))))
 
 (define (report-display value)
   (display value)
